/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/



// CCustomerView.cpp : implementation file
//
#include "StdAfx.h"
#include "Resource.h"
#include "CustomerView.h"
#include "ULFoundationAPI.hpp"
#include "platform.h"
#include "ULFoundationDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCustomerView

IMPLEMENT_DYNCREATE( CCustomerView, CFormView )

CCustomerView::CCustomerView()
		: CFormView( CCustomerView::IDD )
{
	//{{AFX_DATA_INIT(CCustomer)
	m_no_record_msg = _T( "" );
	//}}AFX_DATA_INIT
	( ( CUlfoundationDoc* ) GetDocument() ) ->customerTable.Open(
							&( ( ( CUlfoundationDoc* ) GetDocument() ) ->conn ) );

	( ( CUlfoundationDoc* ) GetDocument() ) ->customerTable.BeforeFirst();
}

CCustomerView::~CCustomerView()
{
	( ( CUlfoundationDoc* ) GetDocument() ) ->customerTable.Open(
							&( ( ( CUlfoundationDoc* ) GetDocument() ) ->conn ) );
}

void CCustomerView::DoDataExchange( CDataExchange* pDX )
{
	CFormView::DoDataExchange( pDX );
	//{{AFX_DATA_MAP(CCustomerView)
	DDX_Control(pDX, IDC_CUSTOMER_LIST, m_customerList);
	DDX_Text( pDX, IDC_NO_RECORDS, m_no_record_msg );
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP( CCustomerView, CFormView )
//{{AFX_MSG_MAP(CCustomerView)
	ON_COMMAND(FILE_SYNC, OnSync)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCustomerView diagnostics

#ifdef _DEBUG
void CCustomerView::AssertValid() const
{
	CFormView::AssertValid();
}

void CCustomerView::Dump( CDumpContext& dc ) const
{
	CFormView::Dump( dc );
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCustomerView message handlers

void CCustomerView::ClearScreen()
{
   	//set the member variables to the cursor data
	m_no_record_msg = _T( "" );

	//refresh the screen with the new data
	UpdateData( FALSE );
}


void CCustomerView::OnSync() 
{
	CWaitCursor cursor; // starts and stops wait cursor just by instantiating
    bool ok;
    CString msg;
	CUlfoundationDoc * p_doc = ( CUlfoundationDoc* ) GetDocument();

	p_doc->conn.InitSynchInfo( &( p_doc->synch_info ) );

	p_doc->synch_info.user_name = UL_TEXT( "user23" );
	p_doc->synch_info.version = UL_TEXT( "1.0" );
	p_doc->synch_info.stream = ULSocketStream();

	// Pocket PC 2002 Emulator synchronization fix goes here
	p_doc->synch_info.stream_parms = NULL;

	ok = p_doc->conn.Synchronize( &( p_doc->synch_info ) );
	if( ok ) {
		msg = _T( "Synchronization was successful." );
		AfxMessageBox( msg, MB_OK | MB_ICONINFORMATION  );
	} else {
		msg.Format( _T( "Synchronization returned with an error: [%ld]"),
					p_doc->m_uldb.getSQLCode() );
		AfxMessageBox( msg, MB_OK | MB_ICONERROR );
	}

	ClearScreen();
}

void CCustomerView::PopulateCustomerList(void)
{
	m_customerList.DeleteAllItems();

	// attempt to populate the customer list w/ data
	CUlfoundationDoc * p_doc = ( CUlfoundationDoc* ) GetDocument();
	p_doc->customerTable.Open( &( p_doc->conn ) );
	p_doc->customerTable.BeforeFirst();
	long retcode = p_doc->m_uldb.getSQLCode();

	CString msg;
	if ( retcode != SQLE_NOERROR ) {
		msg.Format( _T( "Opening cursor returned an error: [%ld]"), retcode );
		AfxMessageBox( msg, MB_OK | MB_ICONERROR );
	}

	bool hasAtLeastOne = false;
	int index = 0;

	while ( p_doc->customerTable.Next() ) {
		bool isNull;
		CString str;
		TCHAR nameBuf[30];

		// set the id
		long id;
		p_doc->customerTable.GetCustomer_id( id, & isNull );
		// TODO: should not ignore isNull value
		str.Format( _T("%i"), id );
		int ret = m_customerList.InsertItem( LVIF_TEXT, index, str, 0, -1, -1, NULL );

		// set the first name
		p_doc->customerTable.GetFirst_name( nameBuf, 30, & isNull );
		// TODO: should not ignore isNull value
		m_customerList.SetItem( ret, 1, LVIF_TEXT, nameBuf, -1, -1, -1, NULL );

		// set the last name
		p_doc->customerTable.GetLast_name( nameBuf, 30, & isNull );
		m_customerList.SetItem( ret, 2, LVIF_TEXT, nameBuf, -1, -1, -1, NULL );

		// update for next loop iteration
		hasAtLeastOne = true;
		index++;
	}

	if( !hasAtLeastOne ) {
		m_no_record_msg = _T( "No records to fetch, you might " );
		m_no_record_msg += _T( "try synchronizing." );
		UpdateData( FALSE );
	} else {
		m_no_record_msg = _T( "" );
		UpdateData( FALSE );
	}

	p_doc->customerTable.Close();
	retcode = p_doc->m_uldb.getSQLCode();

	if ( retcode != SQLE_NOERROR ) {
		msg.Format( _T( "Closing cursor returned an error: [%ld]"), retcode );
		AfxMessageBox( msg, MB_OK | MB_ICONERROR );
	}
}


void CCustomerView::SetCustomerListHeader(void) 
{
	m_customerList.InsertColumn( 0, _T("ID"), LVCFMT_LEFT, 25 );
	m_customerList.InsertColumn( 1, _T("FIRST NAME"), LVCFMT_LEFT, 80);
	m_customerList.InsertColumn( 2, _T("LAST NAME"), LVCFMT_LEFT, 80);
}


void CCustomerView::OnInitialUpdate() 
{
	CFormView ::OnInitialUpdate();
	
	SetCustomerListHeader();
}

void CCustomerView::OnDraw(CDC* pDC) 
{
	ClearScreen();
	PopulateCustomerList();
}


/*
void CCustomerView::OnDisplayButton()
{
	customer_struct customer;

	CUlfoundationDoc* p_doc = ( CUlfoundationDoc* ) GetDocument();

	//get the next entry in the database cursor
	p_doc->customerTable.Next();

	long retcode = p_doc->conn.GetSQLCode();

	if( SQLE_NOTFOUND == retcode ) {  //no more records in cursor
		OnNoRecords();

	} else {
		//get customer info from cursor
		p_doc->customerTable.GetCustomer_id( customer.customer_id );
		p_doc->customerTable.GetFirst_name( customer.first_name,
											FIRST_NAME_LENGTH );
		p_doc->customerTable.GetLast_name( customer.last_name,
										   LAST_NAME_LENGTH );

		//set the member variables to the cursor data
		m_no_record_msg = _T( "" );
		m_id.Format( _T( "%i" ), customer.customer_id );
		m_first_name = customer.first_name;
		m_last_name = customer.last_name;

		//refresh the screen with the new data
		UpdateData( FALSE );
	}
}

void CCustomerView::OnNoRecords()
{
	m_no_record_msg = _T( "No more records to fetch.  You might" );
	m_no_record_msg += _T( " try synchronizing to reset the cursor." );

	//the values on the page are being initialized to the member variable values
	UpdateData( FALSE ); 
}
*/
