/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.sql.*;
import java.util.Properties;
import ianywhere.ultralite.jdbc.*;
import ianywhere.ultralite.runtime.*;
import ianywhere.ultralite.ui.SynchProgressViewer;


/**
 * A frame which contains the ULFoundation application. This is a very
 * simple UltraLite application which shows how to use the Java API.
 *
 * @author <a href="http://www.ianywhere.com">iAnywhere Solutions</a>
 * @version 1.0
 */

public class ULFoundation extends Frame implements ActionListener {

    /**
     * The text label for the 'Synchronize' button.
     */
    public static final String SYNCH_LABEL = "Synchronize";

    /**
     * The text label for the 'Help' drop-down menu.
     */
    public static final String HELP_LABEL = "Help";

    /**
     * The text label for the 'About' option in the Help drop-down.
     */
    public static final String ABOUT_LABEL = "About";

    /**
     * The title that appears at the top of the main frame of the
     * application.
     */
    public static final String APP_TITLE = "UltraLite Foundation Application";

    /**
     * The name of the JDBC Driver to use to create the connection to
     * the UltraLite database.
     */
    private static final String DRIVER_NAME =
	"ianywhere.ultralite.jdbc.JdbcDriver";

    /**
     * The JDBC URL to use to connect to the UltraLite database.
     */
    private static final String JDBC_URL = "jdbc:ultralite:ULDB";

    /**
     * The column number of the ID column in the Customer table.
     */
    private static final int ID_COL = 1;

    /**
     * The column number of the First Name column in the Customer
     * table.
     */
    private static final int FIRST_NAME_COL = 2;

    /**
     * The column number of the Last Name column in the Customer
     * table.
     */
    private static final int LAST_NAME_COL = 3;

    /**
     * The username with which to synchronize to the consolidated
     * database.
     */
    private static final String SYNCH_USERNAME = "user23";

    /**
     * The synchronization script version number to use when
     * synchronizing to the consolidated database.
     */
    private static final String SYNCH_VER = "1.0";

    /**
     * The synchronization parameters to use when synchronizing to the
     * consolidated database.
     */
    private static final String SYNCH_PARMS = "host=localhost";

    /**
     * The UltraLite database connection object.
     */
    private JdbcConnection conn;

    /**
     * The Customer table object from the UltraLite Database.
     */
    private PreparedStatement customerStmt;

    /**
     * The cursor with which we iterate through the Customer table.
     */
    private ResultSet tableCursor;

    /**
     * The List holding the data from the cursor. 
     */
    private List tableList;

    /**
     * The drop-down menu which appears at the top of the screen.
     */
    private MenuBar ulFoundationMenuBar = new MenuBar();

    /**
     * The Help drop-down menu which is contained in the menubar.
     */
    private Menu helpMenu = new Menu( HELP_LABEL );

    /**
     * The About menu choice which is contained in the Help menu.
     */
    private MenuItem about = new MenuItem( ABOUT_LABEL );

    /**
     * Constructor. Creates a new instance of the ULFoundation main
     * frame.
     *
     * @param _conn an existing connection to a database
     */
    public ULFoundation( JdbcConnection _conn ) {
	super( APP_TITLE );

	conn = _conn;

	try {

	    // Instantiate a copy of our Customer table.
	    customerStmt = conn.prepareStatement( SQL.CUSTOMER_QUERY );

	    // Create a cursor which will iterate through the Customer table.
	    tableCursor = customerStmt.executeQuery();

	} catch( SQLException e ) {
	    e.printStackTrace();
	}

	initComponents();
	populateList();
    }


    /**
     * Lays out the components of this Frame.
     */
    private void initComponents() {

        ScrollPane scrollPane1 = new ScrollPane();
        Panel panelOuter = new Panel();
	tableList = new List();
	Button buttonSynch = new Button( SYNCH_LABEL );

	setBackground( java.awt.Color.white );
	setFont( new Font( "SansSerif", Font.PLAIN, 14 ) );

	// Handles the closing of this window.
	addWindowListener( new WindowAdapter() {
		public void windowClosing( WindowEvent e ) {
		    setVisible( false );
		    dispose();
		    System.exit( 0 );
		}
	    } );

	buttonSynch.addActionListener( this );

        panelOuter.setLayout(new BorderLayout());
        panelOuter.add( tableList, BorderLayout.CENTER );
	panelOuter.add( buttonSynch, BorderLayout.SOUTH );

        scrollPane1.add( panelOuter );
	add( scrollPane1 );

        // construct the menu for the about dialog
	Menu m = new Menu("Options");
	m.add( about );
	MenuBar bar = new MenuBar();
	bar.add( m );
	setMenuBar( bar );
	about.addActionListener( this ); 


        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new Dimension(250, 225));
        setLocation((screenSize.width-250)/2,(screenSize.height-225)/2);
    }


    /**
     * Handles any events which this frame registered for. This method
     * is needed to implement the <code>ActionListener</code>
     * interface.
     *
     * @param ae The <code>ActionEvent</code> received by this frame.
     */
    public void actionPerformed( ActionEvent ae ) {

	Object actionEvent = ae.getSource();

	if( actionEvent instanceof Button ) {
	    Button buttonEvent = ( Button ) actionEvent;

	    if( buttonEvent.getLabel().equals( SYNCH_LABEL ) ) {

		// Synchronize with the consolidated database via
		// MobiLink.
		synchDatabase();

		// Populate the List with data for display
		populateList();
		return;
	    }
	} else if( actionEvent instanceof MenuItem ) {
	    MenuItem menuEvent = ( MenuItem ) actionEvent;

	    if( menuEvent.getLabel().equals( ABOUT_LABEL ) ) {

		// Display a rudimentary "About" box.
		Dialog aboutULFoundation =
		    new ULFoundationDialog( this,
					    "About ULFoundation",
					    true,
					    "UltraLite Foundation Application"
					    + "Version 2.0\n"
					    + "for "
					    + "Java version 1.1.8+" );
		aboutULFoundation.show();
		return;
	    }
	}
    }

    /**
     * Synchronizes the UltraLite database with the remote
     * consolidated database.
     */
    private void synchDatabase() {
	// Create a UlSynchOptions Object, and populate it with the
	// necessary synchronization information.
	UlSynchOptions opts = new UlSynchOptions();
	SynchProgressViewer viewer = new SynchProgressViewer( this );

	opts.setUserName( SYNCH_USERNAME );
	opts.setScriptVersion( SYNCH_VER );
	opts.setStream( new UlSocketStream() );
	opts.setStreamParms( SYNCH_PARMS );

	Cursor waitCursor = Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR );

	// Perform the actual synchronization using the options we
	// set.

	((Component)this).setCursor( waitCursor );
	boolean status = viewer.synchronize( conn, opts );
	((Component)this).setCursor( null );

	if ( !status ) {
	    //viewer.synchronize returns false for success
	    Dialog success = new ULFoundationDialog( this,
						     "Success",
						     true,
						     "Synchronization completed"
						     + " successfully." );
	    success.show();
	} else {
	    Dialog error = new ULFoundationDialog( this, 
			                             "Error",
						     true, 
						     "Synchronization failed.");
	    error.show();
	}
    }

    /**
     * Adds the data held by the cursor to the List for display. 
     */
    private void populateList() {

	try {
	    // Open the table cursor to retrieve the customer data from
	    // the ultralite database
	    tableCursor = customerStmt.executeQuery();
            
            tableList.removeAll(); // clear List contents
	    String tab = "      ";
	    tableList.add("ID" + tab + "NAME"); // add the header to the List
            while ( tableCursor.next() ) {
	        String idStr = tableCursor.getString( ID_COL );
	        String nameStr = tableCursor.getString( FIRST_NAME_COL )
				+ " "
				+ tableCursor.getString( LAST_NAME_COL );
		tableList.add( idStr + tab + nameStr );
	    }

	} catch (SQLException e ) {
	    String msg = "An SQL error occurred fetching customer data.";
	    System.out.println( msg );
	    e.printStackTrace();
	} finally {
	    try {    
	        // Always close the table Cursor 
	        tableCursor.close();
	    } catch( SQLException e ) {
	        String msg = "An SQL error occurred closing the Customer cursor.";
	        System.out.println( msg );
	        e.printStackTrace();
            }
	}
    }


    /**
     * This main method allows this java file to be compiled and run
     * as-is to demonstrate the capabilities of the ULFoundation
     * application.
     *
     * @param args[] The command-line arguments passed in to this
     * application.
     */
    public static void main( String args[] ) {

	try {

	    try {
		// We must instantiate the JDBC Driver before trying
		// to create a connection.
		Class.forName( DRIVER_NAME );
	    } catch( ClassNotFoundException e ) {
		System.out.println( "The class named "
				    + DRIVER_NAME
				    + " was not found. Please check that it"
				    + " is located in your classpath and"
				    + " try again." );
		e.printStackTrace();
	    }

	    Properties props = new Properties();

	    // This will cause the database to be saved to disk as
	    // opposed to remaining in memory and being recreated
	    // every time the application runs.
	    props.put( "persist", "file" );

	    // This is the default name of the file in which to save
	    // our database info.
	    props.put( "persistfile", "database.udb" );

	    try {
		// We need to do this before connecting to the
		// database for the first time.
		new ULDB( props );

		JdbcConnection new_conn =
		    (JdbcConnection)DriverManager.getConnection(JDBC_URL,props);

		// Create a new instance of our application's frame,
		// and make it visible.
		ULFoundation ulf = new ULFoundation( new_conn );
		ulf.setVisible( true );
	    } catch( SQLException e ) {
		String msg =
		    "An SQL error occurred creating the database connection.";
		System.out.println( msg );
		e.printStackTrace();
	    }

	} catch( Throwable t ) {
	    System.out.println( "An unknown error was encountered." );
	    t.printStackTrace();
	    System.exit( 1 );
	}
    }
}
