/*##COPYRIGHT
***********************************************************************
*       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
*       All rights reserved. iAnywhere Solutions is a trademark       *
*       of Sybase, Inc.  No part of this software may be              *
*       reproduced in any form or by any means - graphic,             *
*       electronic or mechanical, including photocopying,             *
*       recording, taping or information storage and retrieval        *
*       systems - except with the written permission of               *
*       iAnywhere Solutions, Inc.                                     *
*                                                                     *
***********************************************************************
#*/

import java.awt.*;
import java.awt.event.*;

/**
 * This class is somewhat limiting, as you must recreate an object just
 * to change the message. However, it does support multi-line messages.
 */
class ULFoundationDialog extends Dialog implements ActionListener {

    /**
     * An internal reference the parent frame of this dialog.
     */
    Frame parentFrame;

    /**
     * The OK Button which appears on the dialog.
     */
    Button btnOK;

    /**
     * Creates a new dialog box which contains the given message.
     *
     * @param parent The parent frame to which this dialog box is
     * linked.
     *
     * @param title The title of the dialog box.
     *
     * @param modal Whether or not this dialog should be modal.  true
     * will make this modal to the parent frame, false will not.
     *
     * @param msg The message to display in the dialog box. To create
     * a multiline message, use '\n' as the end-of-line character.
     */
    public ULFoundationDialog( Frame parent, String title, boolean modal,
			       String msg ) {

	super( parent, modal );

	parentFrame = parent;

	setVisible( false );

	setTitle( title );

	Panel panel = new Panel();
	panel.setLayout( new GridBagLayout() );
	add( panel );

	btnOK = new Button( "OK" );
	btnOK.addActionListener( this );

	InnerMultilineLabel label = new InnerMultilineLabel( msg );

	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;
	panel.add( label, c );

	c.fill = GridBagConstraints.NONE;
	c.gridy = 1;
	panel.add( btnOK, c );

	pack();
	centerInParent();
    }

    /**
     * Centers this dialog box in the parent frame.
     */
    public void centerInParent() {
	Frame parent = (Frame)getParent();
	Dimension dim = parent.getSize();
	Point     loc = parent.getLocationOnScreen();

	Dimension size = getSize();

	loc.x += (dim.width  - size.width)/2;
	loc.y += (dim.height - size.height)/2;

	if (loc.x < 0) loc.x = 0;
	if (loc.y < 0) loc.y = 0;

	Dimension screen = getToolkit().getScreenSize();

	if (size.width  > screen.width)
	    size.width  = screen.width;
	if (size.height > screen.height)
	    size.height = screen.height;

	if (loc.x + size.width > screen.width)
	    loc.x = screen.width - size.width;

	if (loc.y + size.height > screen.height)
	    loc.y = screen.height - size.height;

	setBounds(loc.x, loc.y, size.width, size.height);
    }

    /**
     * Handles any events which this frame registered for. This method
     * is needed to implement the <code> ActionListener </code>
     * interface.  Since the only action performed on this dialog box
     * will be either the pressing of the OK button or the top-right
     * close button, we can just make this dialog invisible when an
     * action is received.
     *
     * @param ae The <code>ActionEvent</code> received by this frame.
     */
    public void actionPerformed( ActionEvent ae ) {
	setVisible( false );
    }
}
