/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/
   


#include "FirstForm.h"


// Row & Col index constants
//
#define COL_CUST_ID    0
#define COL_CUST_FNAME 1
#define COL_CUST_LNAME 2
#define ROW_HEADER     0

#define ROWS 6
#define COLS 3

// variable used to store the TablePtr resource
//
TablePtr m_pTable;


void FirstFormInit( FormPtr pForm )
{
	if( db.Reopen() ) {
		if( !conn.Reopen( &db ) ) {
			FrmCustomAlert( DebugAlert, "Connection is NOT Open!", 
							NULL, NULL );
		}
	} else {
		FrmCustomAlert( DebugAlert, "Database is NOT Open!", NULL, NULL );
	}


	// initialize the customer table 
	m_pTable = (TablePtr)FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, 
                                                   FirstCustomersTable));
	for ( int r=0; r < ROWS; r++ ) {
		for ( int c=0; c < COLS; c++ ) {
			int textLen = ColTextLength( c );
			TCHAR * pTextPtr = (TCHAR * )MemPtrNew( textLen );
			TblSetItemPtr(m_pTable, r, c, pTextPtr);
            TblSetItemStyle(m_pTable, r, c, customTableItem);
			TblSetCustomDrawProcedure(m_pTable, c, DrawTableCell );
            TblSetColumnUsable(m_pTable, c, true /* isUsable */ );
		}
		
		if ( r == 0 ) {
            TblSetRowSelectable( m_pTable, r, false /* isSelectable */ );
        } else { 
        	TblSetRowSelectable( m_pTable, r, true  /* isSelectable */ );
        }
        TblSetRowUsable(m_pTable, r, false /* isUsable */ );
	}
	
	PopulateTable( pForm );

}


void FirstFormFini( FormPtr pForm )
{
	// free the text pointers used by the TablePtr resource
	for ( int r=0; r < ROWS; r++ ) {
		for ( int c=0; c < COLS; c++ ) {
			TCHAR * pTextPtr = (TCHAR *)TblGetItemPtr( m_pTable, r, c );
			MemPtrFree( pTextPtr );
		}
	}
}


Boolean FirstFormHandleMenu( Word command )
{
	Boolean handled = false;

	switch( command ) {

	case FirstOptionsAboutULFoundation:
		MenuEraseStatus( 0 );
		FrmAlert( AboutAlert );
		handled = true;
		break;
	}
	return handled;
}


Boolean FirstFormHandleEvent( EventPtr pEvent )
{
	FormPtr pForm;

	Boolean handled = false;

	pForm = FrmGetActiveForm();

	switch( pEvent->eType ) {

	case menuEvent:    // user selected a menu item
		// Call our menu-hanlding function
		return FirstFormHandleMenu( pEvent->data.menu.itemID );

	case frmOpenEvent:    // form was just opened
		FirstFormInit( pForm );
		FrmDrawForm( pForm );
		handled = true;
		break;

	case frmCloseEvent:   // form is closing
		FirstFormFini( pForm );
		break;

	case appStopEvent:
		customerTable.Close();
		break;

	case ctlSelectEvent:   // user clicked on a form button

		handled = true;

		switch( pEvent->data.ctlSelect.controlID ) {

		case FirstSynchronizeButton:
				Synchronize( pForm );
			break; 
		}
		break;
	}
	return handled;
}


void Synchronize( FormPtr pForm )
{
	// We are going to synchronize with the consolidated database.

	// Tell the database to synchronize.
	conn.InitSynchInfo( &synch_info );

	synch_info.user_name = UL_TEXT( "user23" );
	synch_info.version = UL_TEXT( "1.0" );
	synch_info.stream = ULSocketStream();
	synch_info.stream_parms = NULL;

	conn.Synchronize( &synch_info );

	long retcode = conn.GetSQLCode();

	TCHAR msg[ ERROR_MSG_LEN + 1 ];
	if( SQLE_NOERROR != retcode ) {
		StrPrintF( msg, "Synchronization returned with an error: [%ld]",
				   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	} else {
		StrPrintF( msg, "Synchronization completed successfully." );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	}

	PopulateTable( pForm );
}

void PopulateTable( FormPtr pForm )
{
	// set the header row for the Table
	TCHAR * pTextPtr;
	int len;
		
	pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, ROW_HEADER, COL_CUST_ID);
	len = StrPrintF( pTextPtr, "ID" );
	//pTextPtr[len] = '\0';
	pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, ROW_HEADER, COL_CUST_FNAME);
	len = StrPrintF( pTextPtr, "FIRST NAME" );
	//pTextPtr[len] = '\0';
	pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, ROW_HEADER, COL_CUST_LNAME);
	len = StrPrintF( pTextPtr, "LAST NAME" );
	//pTextPtr[len] = '\0';
	TblSetRowUsable(m_pTable, ROW_HEADER, true /* isUsable */ );

	
	// We need to open the customer table to populate the TablePtr 
	//
	customerTable.Open( &conn );
	long retcode = conn.GetSQLCode();

	TCHAR msg[ ERROR_MSG_LEN + 1 ];
	if( SQLE_NOERROR != retcode ) {
		StrPrintF( msg, "Error opening customer cursor [%ld]",
			   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	} else { 
		// no error, so populate table w/ data
		int r = ROW_HEADER + 1;
		int c = 0;
		
		bool hasAtLeastOne = false;
		customer_struct customer;
		customerTable.Next();
		retcode = conn.GetSQLCode();
		
		while ( retcode == SQLE_NOERROR ) {
			
			// pull data from database	
			customerTable.GetCustomer_id( customer.customer_id );
			customerTable.GetFirst_name( customer.first_name, FIRST_NAME_LENGTH );
			customerTable.GetLast_name( customer.last_name, LAST_NAME_LENGTH );
			
			// populate table cells w/ customer data
			pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, r, c++);
			len = StrPrintF( pTextPtr, "%ld", customer.customer_id );
			pTextPtr[len] = '\0';
			pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, r, c++);
			len = StrPrintF( pTextPtr, "%s", customer.first_name );
			pTextPtr[len] = '\0';
			pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, r, c++);
			len = StrPrintF( pTextPtr, "%s", customer.last_name );
			pTextPtr[len] = '\0';
			TblSetRowUsable(m_pTable, r, true /* isUsable */ );
			
			customerTable.Next();
			retcode = conn.GetSQLCode();
			r++;
			c = 0;
			hasAtLeastOne = true;
		}
		
		if ( !hasAtLeastOne ) {
			// We found no records, so the user should synchronize the database
			//
			FrmCustomAlert( DebugAlert, "No records to fetch.", NULL,
							"You might try synchronizing the database." );
		}
	}
			
	customerTable.Close();
	retcode = conn.GetSQLCode();
	
	if ( retcode != SQLE_NOERROR ) {
		StrPrintF( msg, "Error closing customer cursor [%ld]",
			   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	}
	
	TblMarkTableInvalid(m_pTable);
	TblRedrawTable(m_pTable);
}



int ColTextLength( int col )
{
	int textLen = 1;
	if ( col == COL_CUST_ID ) {
		textLen += ID_NUMBER_LEN;
	} else if ( col == COL_CUST_FNAME ) {
		textLen += FIRST_NAME_LEN;
	} else if ( col == COL_CUST_LNAME ) {
		textLen += LAST_NAME_LEN;
	} else {
		// not a valid column index, return -1
		textLen = -1;
	}
	return textLen;
}



void DrawTableCell( void * pTable, Int16 row, Int16 col, RectangleType *bounds )
{
	TCHAR * pText = (TCHAR*)TblGetItemPtr( (TablePtr)pTable, row, col );
	WinDrawChars( pText, StrLen( pText ), bounds->topLeft.x, bounds->topLeft.y );
}




/*
void FirstFormDisplayNextCustomer( FormPtr pForm )
{ 	
	customer_struct customer;
	TCHAR msg[ ERROR_MSG_LEN + 1 ];
	long retcode;

	// Fetch the next customer record from the cursor.  Note that this
	// assumes that the cursor is already open.
	//
	customerTable.Next();

	retcode = conn.GetSQLCode();

	if( SQLE_NOTFOUND == retcode ) {

		// We found no more records.  This means we've traversed the entire
		// result set from this cursor.
		//
		// You might want to add a "Reset Cursor" button to this form
		// which would simply close and re-open the cursor.
		//
		FrmCustomAlert( DebugAlert, "No more records to fetch.", NULL,
						"You might try synchronizing to reset the cursor." );
		return;

	} else if( SQLE_NOERROR != retcode ) {
		// Some unexpected error occured.
		//
		StrPrintF( msg, "FirstFormDisplayNextCustomer: fetch returned [%ld]",
				   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
		return;
	}

	// We successfully fetched the new customer record.  Now let's
	// display it.
	//
	customerTable.GetCustomer_id( customer.customer_id );

	customerTable.GetFirst_name( customer.first_name, FIRST_NAME_LENGTH );

	customerTable.GetLast_name( customer.last_name, LAST_NAME_LENGTH );

	StrPrintF( msg, "%ld", customer.customer_id );

	PalmSetFieldText( handleIdNumber, FirstIdNumberField, msg );

	PalmSetFieldText( handleFirstName, FirstFirstNameField,
                      customer.first_name );

	PalmSetFieldText( handleLastName, FirstLastNameField,
                      customer.last_name );
    
    // set text field usage flag to 'true' since they've been used
    textFieldsUsed = 1;
}






	// Open the database cursor so we can retrieve customer records
	//
	customerTable.Open( &conn );

	long retcode = conn.GetSQLCode();

	switch( retcode ) {

	case SQLE_NOERROR:
		// Everything is okay!
		break;

	case SQLE_NOTFOUND:
		FrmCustomAlert( DebugAlert, "Database contains no customers",
				NULL, "You might try synchronizing?" );
		break;

	default:
		TCHAR msg[ ERROR_MSG_LEN + 1 ];
		StrPrintF( msg, "ERROR: OpenCustomerCursor() returned [%ld]",
			   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
		break;
	}

*/