/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/




#include "FirstForm.h"


// Row & Col index constants
//
#define COL_CUST_ID    0
#define COL_CUST_FNAME 1
#define COL_CUST_LNAME 2
#define ROW_HEADER     0

#define ROWS 6
#define COLS 3

// variable used to store the TablePtr resource
//
TablePtr m_pTable;


void FirstFormInit( FormPtr pForm )
{
	// initialize the customer table 
	m_pTable = (TablePtr)FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, 
                                                   FirstCustomersTable));
	for ( int r=0; r < ROWS; r++ ) {
		for ( int c=0; c < COLS; c++ ) {
			int textLen = ColTextLength( c );
			TCHAR * pTextPtr = (TCHAR * )MemPtrNew( textLen );
			TblSetItemPtr(m_pTable, r, c, pTextPtr);
            TblSetItemStyle(m_pTable, r, c, customTableItem);
			TblSetCustomDrawProcedure(m_pTable, c, DrawTableCell );
            TblSetColumnUsable(m_pTable, c, true /* isUsable */ );
		}
		
		if ( r == 0 ) {
            TblSetRowSelectable( m_pTable, r, false /* isSelectable */ );
        } else { 
        	TblSetRowSelectable( m_pTable, r, true  /* isSelectable */ );
        }
        TblSetRowUsable(m_pTable, r, false /* isUsable */ );
	}
	
	PopulateTable( pForm );
}


void FirstFormFini( FormPtr pForm )
{
	// free the text pointers used by the TablePtr resource
	for ( int r=0; r < ROWS; r++ ) {
		for ( int c=0; c < COLS; c++ ) {
			TCHAR * pTextPtr = (TCHAR *)TblGetItemPtr( m_pTable, r, c );
			MemPtrFree( pTextPtr );
		}
	}
}


Boolean FirstFormHandleMenu( Word command )
{
	Boolean handled = false;

	switch( command ) {

	case FirstOptionsAboutULFoundation:
		MenuEraseStatus( 0 );
		FrmAlert( AboutAlert );
		handled = true;
		break;
	}
	return handled;
}


Boolean FirstFormHandleEvent( EventPtr pEvent )
{
	FormPtr pForm;
	Boolean handled = false;

	pForm = FrmGetActiveForm();

	switch( pEvent->eType ) {

	case menuEvent:    // user selected a menu item
		// Call our menu-hanlding function
		return FirstFormHandleMenu( pEvent->data.menu.itemID );

	case frmOpenEvent:    // form was just opened
		FirstFormInit( pForm );
		FrmDrawForm( pForm );
		handled = true;
		break;

	case frmCloseEvent:   // form is closing
		FirstFormFini( pForm );
		break;

	case ctlSelectEvent:   // user clicked on a form button

		handled = true;

		switch( pEvent->data.ctlSelect.controlID ) {

		case FirstSynchronizeButton:
				Synchronize( pForm );
			break;
		}
		break;
	}
	return handled;
}


void Synchronize( FormPtr pForm )
{
	// We are going to synchronize with the consolidated database.

	// Tell the database to synchronize.  It uses whatever parameters
	// we have set (currently we only set them when we call "new
	// ULDB()" in AppStart()  [see Main.cpp]
	//
	long retcode = pULDB->Synchronize();

	TCHAR msg[ ERROR_MSG_LEN + 1 ];
	if( SQLE_NOERROR != retcode ) {
		StrPrintF( msg, "Synchronization returned with an error: [%ld]",
				   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	} else {
		StrPrintF( msg, "Synchronization completed successfully." );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	}

	PopulateTable( pForm );
}


void PopulateTable( FormPtr pForm )
{
	// set the header row for the Table
	TCHAR * pTextPtr;
	int len;
		
	pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, ROW_HEADER, COL_CUST_ID);
	len = StrPrintF( pTextPtr, "ID" );
	//pTextPtr[len] = '\0';
	pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, ROW_HEADER, COL_CUST_FNAME);
	len = StrPrintF( pTextPtr, "FIRST NAME" );
	//pTextPtr[len] = '\0';
	pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, ROW_HEADER, COL_CUST_LNAME);
	len = StrPrintF( pTextPtr, "LAST NAME" );
	//pTextPtr[len] = '\0';
	TblSetRowUsable(m_pTable, ROW_HEADER, true /* isUsable */ );

	// We need to open the cursor to retrieve the customer data for the 
	// TablePtr resource
	//
	long retcode = pULDB->OpenCustomerCursor();

	TCHAR msg[ ERROR_MSG_LEN + 1 ];
	if( SQLE_NOERROR != retcode ) {
		StrPrintF( msg, "Error opening customer cursor [%ld]",
			   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	} else { 
		// no error, so populate table w/ data
		int r = ROW_HEADER + 1;
		int c = 0;
		
		bool hasAtLeastOne = false;
		customer_struct customer;
		retcode = pULDB->GetNextCustomerRecord( &customer );
		
		while ( retcode == SQLE_NOERROR ) {
			
			// populate table cells w/ customer data
			pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, r, c++);
			len = StrPrintF( pTextPtr, "%ld", customer.customer_id );
			pTextPtr[len] = '\0';
			pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, r, c++);
			len = StrPrintF( pTextPtr, "%s", customer.first_name );
			pTextPtr[len] = '\0';
			pTextPtr = (TCHAR*)TblGetItemPtr(m_pTable, r, c++);
			len = StrPrintF( pTextPtr, "%s", customer.last_name );
			pTextPtr[len] = '\0';
			TblSetRowUsable(m_pTable, r, true /* isUsable */ );
			
			retcode = pULDB->GetNextCustomerRecord( &customer );
			r++;
			c = 0;
			hasAtLeastOne = true;
		}
		
		if ( !hasAtLeastOne ) {
			// We found no records, so the user should synchronize the database
			//
			FrmCustomAlert( DebugAlert, "No records to fetch.", NULL,
							"You might try synchronizing the database." );
		}
	}
			
	retcode = pULDB->CloseCustomerCursor();
	
	if ( retcode != SQLE_NOERROR ) {
		StrPrintF( msg, "Error closing customer cursor [%ld]",
			   retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );
	}
	
	TblMarkTableInvalid(m_pTable);
	TblRedrawTable(m_pTable);
}



int ColTextLength( int col )
{
	int textLen = 1;
	if ( col == COL_CUST_ID ) {
		textLen += ID_NUMBER_LEN;
	} else if ( col == COL_CUST_FNAME ) {
		textLen += FIRST_NAME_LEN;
	} else if ( col == COL_CUST_LNAME ) {
		textLen += LAST_NAME_LEN;
	} else {
		// not a valid column index, return -1
		textLen = -1;
	}
	return textLen;
}



void DrawTableCell( void * pTable, Int16 row, Int16 col, RectangleType *bounds )
{
	TCHAR * pText = (TCHAR*)TblGetItemPtr( (TablePtr)pTable, row, col );
	WinDrawChars( pText, StrLen( pText ), bounds->topLeft.x, bounds->topLeft.y );
}
