/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/




/***********************************************************************
 *
 * PROJECT:  ULFoundation
 * FILE:     Main.c
 * AUTHOR:   iAnywhere Solutions, Applications Development
 *
 * DECLARER: Main
 *
 * DESCRIPTION: This file contains the PilotMain() for the ULFoundation
 *              project.
 *
 *              ULFoundation is a minimum project which demonstrates an
 *              UltraLite program for Palm OS and which uses the
 *              iAnywhere Solutions' PalmCommon libraries.
 *              
 **********************************************************************/
#if UL_PALMOS_SDK < 35
// For use with PalmOS SDK versions earlier than 3.5 (default)
#include <Pilot.h>

#else
// For use with PalmOS SDK 3.5 and above
#include <PalmOS.h>
#include <PalmCompatibility.h>

#if UL_PALMOS_SDK >= 40
// For use with PalmOS SDK 4.0 and above
#include <PalmUtils.h>
#endif

#endif

#include <SysEvtMgr.h>
#include "Main.h"

/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/

// a pointer to our UltraLite Database object
ULDB *pULDB = NULL;


static Boolean AppHandleEvent( EventPtr pEvent )
{
	Word formId;
	FormPtr pForm;

	if( pEvent->eType == frmLoadEvent ) {

		// Load the form resource.
		formId = pEvent->data.frmLoad.formID;
		pForm = FrmInitForm( formId );
		FrmSetActiveForm( pForm );

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch( formId ) {

		case FirstForm:
			FrmSetEventHandler( pForm, FirstFormHandleEvent );
			break;

		default:
			// This is not necessary, but it is left to demonstrate the
			// use of FrmCustomAlert().  DebugAlert is in ULFoundation.rsrc
			//
			TCHAR msg[ ERROR_MSG_LEN + 1 ];
			StrPrintF( msg, "AppHandleEvent: invalid form load [%ld]",
					   formId );
			FrmCustomAlert( DebugAlert, msg, NULL, NULL );
			break;
		}
		return true;  // we handled this event
	}
	return false;  // we did not handle the event
}


static void AppEventLoop( void )
{
	Word error;
	EventType event;

	// This loop is standard in most Palm OS applications and should
	// be fine as is:
	//       ONLY MODIFY THIS IF YOU KNOW WHAT YOU ARE DOING!!!
	//
	do {
		EvtGetEvent( &event, evtWaitForever );

		if( ! SysHandleEvent( &event ) )
			if( ! MenuHandleEvent( 0, &event, &error ) )
				if( ! AppHandleEvent( &event ) )
					FrmDispatchEvent( &event );

	} while( event.eType != appStopEvent );
}


static Err AppStart( void )
{
	// Instantiate the database object
	pULDB = new ULDB( "user23", "1.0", ULSocketStream(), NULL ); // TCP/IP Synch

	// connect to the UltraLite db
	long retcode = pULDB->StartupDB();

	if( ( DB_STARTUP_SUCCESS != retcode ) &&
		( DB_STARTUP_SUCCESS_FIRST != retcode ) ) {
		TCHAR msg[ ERROR_MSG_LEN + 1 ];

		StrPrintF( msg, "AppStart: StartupDB() returned [%ld]", retcode );
		FrmCustomAlert( DebugAlert, msg, NULL, NULL );

		return 1;
	}
	return 0;
}

static void AppStop( void )
{
    // clear the form from heap storage
    FormPtr formPtr = FrmGetActiveForm();
    FirstFormFini( formPtr );
    FrmEraseForm( formPtr );
    FrmDeleteForm( formPtr );

	if( !pULDB->ShutdownDB() ) {
		FrmCustomAlert( DebugAlert, "ShutdownDB() encountered an error!",
						NULL, NULL );
	}
	
    // delete the UL database from heap storage
    delete pULDB;
}


DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags )
{
	DWord error;

	// Check how we were started and handle appropriately.
	//
	// In ULFoundation, we only care if we were launched normally.  See
	// "Application Launch Codes" in the Palm OS SDK Reference manual.
	//
	switch ( cmd ) {

	case sysAppLaunchCmdNormalLaunch:
		error = AppStart();

		if( error ) {
			return error;
		}
		FrmGotoForm( FirstForm );
		AppEventLoop();
		AppStop();
		break;

	default:
		break;
	}
	return 0;
}
