/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         * 
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/



// ULFoundation.cpp : Defines the entry point for the console application.
//

#include "ULFoundation.h"
#include <string.h>

ULData db;
ULConnection conn;
Customer customerTable;
ul_synch_info synch_info;

int main( int argc, TCHAR* argv[] )
{
	if( FALSE == Initialize() ) {
		ConsolePressAnyKey();
		exit( -1 );  // initialization failed
	}

	DoMainMenu();

	Finalize();

	ConsolePressAnyKey();
	return ( 0 );  // and we're done, without errors!
}


bool Initialize( void )
{
	db.Open();

	if( !conn.Open( &db, "dba", "sql" ) ) {
		printf( "Initialize: ULConnection.Open() returned FALSE.\n" );
		return FALSE;
	}
	return TRUE;
}


void Finalize( void )
{
	if( !conn.Close() ) {
		printf( "\nFinalize: ULConnection.Close() returned FALSE.\n" );
		ConsolePressAnyKey();
	}
}


void DoMainMenu( void )
{
	TCHAR ch = '\0';

	while( ch != 'Q' && ch != 'q' ) {
		PrintMainMenu();

		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch ( ch ) {

		case 'D':
			DoDisplayCustomers();
			break;

		case 'Q':
			if( FALSE == ConsoleYesOrNo( "Really quit" ) ) {
				ch = '\0';  // don't quit, so ch can't be 'Q'
			}
			break;

		case 'S':
		DoSynchronize();
			break;

		default:
			DoInvalidChoice( ch );
			break;
		}
	}
}


void PrintMainMenu( void )
{
	ConsoleClear();
	printf( "Main Menu\n" );
	printf( "---------\n" );
	printf( "(D)isplay Customers\n" );
	printf( "(S)ynchronize the database\n" );
	printf( "\n(Q)uit\n\n" );
	printf( "Choice: " );
	fflush( stdout );
}


void DoSynchronize( void )
{
	long retcode;

	ConsoleClear();
	printf( "Sychronizing...\n" );

	conn.InitSynchInfo( &synch_info );

	synch_info.user_name = UL_TEXT( "user23" );
	synch_info.version = UL_TEXT( "1.0" );
	synch_info.stream = ULSocketStream();
	synch_info.stream_parms = NULL;

	conn.Synchronize( &synch_info );

	retcode = conn.GetSQLCode();

	if( SQLE_NOERROR == retcode ) {
		printf( "\nSynchronization completed successfully.\n\n" );
	} else if ( SQLE_INVALID_LOGON == retcode ) {
		printf( "\nERROR: Bad login id or password.\n\n" );
	} else {
		printf( "\nERROR: synchronization returned: [%d]\n\n", retcode );
	}
	ConsolePressAnyKey();
}


void DoDisplayCustomers( void )
{
	TCHAR ch = '\0';

	while( ch != 'R' && ch != 'r' ) {
		PrintCustomers();
		PrintDisplayCustomersMenu();

		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch( ch ) {

		case 'R':   // return to main menu
			break;

		default:
			DoInvalidChoice( ch );
			break;
		}
	}

/*
	customer_struct customer;
	TCHAR ch = '\0';
	bool is_null = false;

	customerTable.Open( &conn );

	long retcode = conn.GetSQLCode();

	switch( retcode ) {

	case SQLE_NOERROR:
		// Everything is okay!
		break;

	case SQLE_NOTFOUND:
		printf( "\n\t** Database contains no customers\n"
			"\t   You might try synchronizing...\n\n" );
		ConsolePressAnyKey();
		return;

	default:
		printf( "\nERROR: OpenCustomerCursor returned [%ld]\n\n", retcode );
		ConsolePressAnyKey();
		return;
	}
	customerTable.BeforeFirst();

	while( ch != 'R' && ch != 'r' ) {
		PrintDisplayCustomersMenu();

		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch( ch ) {

		case 'N':
			customerTable.Next();
			retcode = conn.GetSQLCode();
			if( SQLE_NOTFOUND == retcode ) {
				// We found no more records.  This means we've traversed the
				// entire result set from this cursor.
				//
				// You might want to add a "Reset Cursor" command in this
				// while loop which would simply close and re-open the
				// cursor.
				//
				printf( "\n\t** No more records to fetch.\n\n"
					"\t   You might try synchronizing to reset the"
					" cursor.\n" );
				ConsolePressAnyKey();
				break;
			} else if( SQLE_NOERROR != retcode ) {
				// Some unexpected error occured.
				//
				printf( "\n\t** ERROR: fetch returned [%ld]\n", retcode );
				ConsolePressAnyKey();
				return;
			}

			// We successfully fetched the new customer record.  Now let's
			// display it.
			//
			customerTable.GetCustomer_id( customer.customer_id );

			customerTable.GetFirst_name( customer.first_name, FIRST_NAME_LENGTH );

			customerTable.GetLast_name( customer.last_name, LAST_NAME_LENGTH );

			PrintCustomerRecord( &customer );

			break;

		case 'R':   // return to main menu
			break;

		default:
			DoInvalidChoice( ch );
			break;
		}
	}
	customerTable.Close();
*/
}

void PrintDisplayCustomersMenu( void )
{
	printf( "\n\n\n\n" );
	printf( "------------------------------------------\n" );
	printf( "\n(R)eturn to Main Menu\n\n" );
	printf( "Choice: " );
	fflush( stdout );
}

void PrintCustomers( void ) 
{
	ConsoleClear();

	printf( "Customer Information\n" );
	printf( "------------------------------------------\n" );

	customer_struct customer;
	bool is_null = false;

	customerTable.Open( &conn );

	long retcode = conn.GetSQLCode();

	switch( retcode ) {

	case SQLE_NOERROR:
		// Everything is okay!
		break;

	case SQLE_NOTFOUND:
		printf( "\n    ** Database contains no customers\n"
			"       You might try synchronizing...\n\n" );
		ConsolePressAnyKey();
		return;

	default:
		printf( "\nERROR: OpenCustomerCursor returned [%ld]\n\n", retcode );
		ConsolePressAnyKey();
		return;
	}
	customerTable.BeforeFirst();

	bool hasAtLeastOne = false;
	customerTable.Next();
	retcode = conn.GetSQLCode();
	while ( retcode == SQLE_NOERROR ) {
		if ( ! hasAtLeastOne ) {
			PrintCustomerHeader();
			hasAtLeastOne = true;
		}

		// We successfully fetched the new customer record.  Now let's
		// display it.
		//
		customerTable.GetCustomer_id( customer.customer_id );
		customerTable.GetFirst_name( customer.first_name, FIRST_NAME_LENGTH );
		customerTable.GetLast_name( customer.last_name, LAST_NAME_LENGTH );
		PrintCustomerRecord( &customer );
		
		customerTable.Next();
		retcode = conn.GetSQLCode();
	}

	if ( retcode != SQLE_NOTFOUND ) {
		// Some unexpected error occured.
		//
		printf( "\n\t** ERROR: fetch returned [%ld]\n", retcode );
		ConsolePressAnyKey();
	}
	
	if ( !hasAtLeastOne ) {
		printf( "\n    ** Database contains no customers\n"
			"       You might try synchronizing..." );
	}

	customerTable.Close();
}

void PrintCustomerHeader( void )
{
	printf( "\n\n\tID\tNAME" );
}

void PrintCustomerRecord( customer_struct *customer )
{
	printf( "\n\n\t" );
	printf( "%ld", customer->customer_id );
	printf( "\t%s ", customer->first_name );
	printf( "%s", customer->last_name );
}

void DoInvalidChoice( TCHAR ch )
{
	// ignore space, tab, newline, etc...
	if( isspace( ch ) ) {
		return;
	}
	printf( "\n\n" );
	printf( "\tInvalid choice: [%c]\n\n", ch );

	ConsolePressAnyKey();
}


// --- Console Helper functions ---
//
void ConsolePressAnyKey( void )
{
	TCHAR ch;

	printf( "\nPress any key to continue..." );
	ch = getch();
	printf( "%c\n\n", ch );
}

bool ConsoleYesOrNo( TCHAR *msg )
{
	TCHAR ch;

	while( 1 ) {
		printf( "\n\n%s (y/n) ? ", msg );
		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch( ch ) {

		case 'N':
			return FALSE;

		case 'Y':
			return TRUE;

		default:
			break;
		}
	}
}


// This function performs the same action as the "cls" DOS command
void ConsoleClear( void )
{
	HANDLE handle;
	CONSOLE_SCREEN_BUFFER_INFO info;
	COORD coord;
	DWORD dwTemp;

	SECURITY_ATTRIBUTES attr;
	attr.nLength = sizeof( attr );
	attr.lpSecurityDescriptor = NULL;
	attr.bInheritHandle = TRUE;

	coord.X = 0;
	coord.Y = 0;

	handle = CreateFile( TEXT( "CONOUT$" ),
			     GENERIC_READ | GENERIC_WRITE,
			     FILE_SHARE_READ | FILE_SHARE_WRITE,
			     &attr,
			     OPEN_EXISTING,
			     ( DWORD ) 0,
			     ( HANDLE ) 0 );

	if( handle != INVALID_HANDLE_VALUE ) {
		if( GetConsoleScreenBufferInfo( handle, &info ) ) {

			// Fill the console with the space character starting from 0,0
			FillConsoleOutputCharacter( handle, ( TCHAR ) ' ',
						    info.dwSize.X * info.dwSize.Y,
						    coord, &dwTemp );
			// Put the cursor back the 0,0 position
			SetConsoleCursorPosition( handle, coord );
		}
		CloseHandle( handle );
	}
}


int ConsoleExit( int exit_code )
{
	ConsolePressAnyKey();
	exit( exit_code );
}
