/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/



#ifndef __ULFOUNDATION_H__
#define __ULFOUNDATION_H__

#include <stdio.h>
#include <Windows.h>
#include <conio.h>
#include "platform.h"
#include "ULFoundationAPI.hpp"


/** The maximum length of a customer's last name.
 */
#define LAST_NAME_LENGTH 30

/** The maximum length of a customer's first name.
 */
#define FIRST_NAME_LENGTH 20

/** A structure which contains information about a given customer.
 */
struct customer_struct {

	/** Represents the customer's ID number.
	 */
	long customer_id;

	/** Represents the customer's first name.
	 */
	ul_char first_name[FIRST_NAME_LENGTH];

	/** Represents the customer's last name.
	 */
	ul_char last_name[LAST_NAME_LENGTH];
};

/** Initializes the UltraLite database.

	This must be called before accessing the database for the first time.
 */
bool Initialize( void );

/** Closes the UltraLite database.

	This must be called before shutting down this application.
 */
void Finalize( void );

/** Draws the main menu and handles input from the console.
 */
void DoMainMenu( void );

/** Does the actual displaying of the main menu. Called from DoMainMenu().
 */
void PrintMainMenu( void );

/** Synchronizes with the consolidated database via MobiLink.
 */
void DoSynchronize( void );

/** Draws the Display Customer screen and handles input from the console.
 */
void DoDisplayCustomers( void );

/** Does the actual displaying of the Display Customer screen. Called from
	DoDisplayCustomers().
 */
void PrintDisplayCustomersMenu( void );

/** Prints out the given customer data to the console. Called from
	PrintDisplayCustomersMenu().

	@param customer A pointer to the customer_struct which contains the
	customer information to be printed.
 */
void PrintCustomerRecord( customer_struct *customer );

/** Does the actual displaying of the customer data. Called from 
    DoDisplayCustomers().
 */
void PrintCustomers( void );

/** Prints out the header for the data to be displayed. 
 */
void PrintCustomerHeader( void );

/** Handles invalid console key choices. Called from DoDisplayCustomers() and
	DoMainMenu().

	@param ch The character which was an invalid choice. If ch is whitespace
	(as defines by the isspace(char) function), this function will do nothing.
	Otherwise, it will print a console message stating that the choice was
	invalid.
 */
void DoInvalidChoice( TCHAR ch );

/** Console helper function which exits the application with a given error
	code.

	@param exit_code The exit code that we wish to return to the invoking
	shell. Normally this is 0 if no errors occur, and nonzero otherwise.

	@return Nothing. This is ignored because the application will exit while
	inside this function.
 */
int  ConsoleExit( int exit_code );

/** Console helper function which pauses until a key is pressed.
 */
void ConsolePressAnyKey( void );

/** Console helper function which clears the console screen.
 */
void ConsoleClear( void );

/** Console helper function which prints a given message and waits for a yes or
	no response. Any response which is not a yes or no response will cause the
	given message to be asked again.

	@param msg A pointer to the message to print.

	@return true if the user responded yes, false if the user responded no.
 */
bool ConsoleYesOrNo( TCHAR *msg );

#endif
