/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2001-2003, by iAnywhere Solutions, Inc.         *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/



// ULFoundation.cpp : Defines the entry point for the console application.
//

#include "ULFoundation.h"

ULDB *pULDB;

int main( int argc, TCHAR* argv[] )
{
	pULDB = NULL;

	if( FALSE == Initialize() ) {
		ConsolePressAnyKey();
		exit( -1 );  // initialization failed
	}
	DoMainMenu();
	Finalize();
	ConsolePressAnyKey();

	return ( 0 );  // and we're done, without errors!
}


bool Initialize( void )
{
	// Instantiate the database object
	pULDB = new ULDB( "user23", "1.0", ULSocketStream(), NULL );  // TCP/IP Synch

	// connect to the UltraLite db
	long retcode = pULDB->StartupDB();

	if( ( DB_STARTUP_SUCCESS != retcode ) &&
		( DB_STARTUP_SUCCESS_FIRST != retcode ) ) {
		printf( "Initialize: StartupDB() returned [%ld]\n", retcode );
		return FALSE;
	}
	return TRUE;
}


void Finalize( void )
{
	unsigned short retcode = pULDB->ShutdownDB();

	if( 0 == retcode ) {
		printf( "\n\t** ERROR: ShutdownDB() returned [%ld]\n", retcode );
		ConsolePressAnyKey();
	}
}


void DoMainMenu( void )
{
	TCHAR ch = '\0';

	while( ch != 'Q' && ch != 'q' ) {
		PrintMainMenu();

		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch ( ch ) {

		case 'D':
			DoDisplayCustomers();
			break;

		case 'Q':
			if( FALSE == ConsoleYesOrNo( "Really quit" ) ) {
				ch = '\0';  // don't quit, so ch can't be 'Q'
			}
			break;

		case 'S':
			DoSynchronize();
			break;

		default:
			DoInvalidChoice( ch );
			break;
		}
	}
}


void PrintMainMenu( void )
{
	ConsoleClear();
	printf( "Main Menu\n" );
	printf( "---------\n" );

	printf( "(D)isplay Customers\n" );
	printf( "(S)ynchronize the database\n" );

	printf( "\n(Q)uit\n\n" );

	printf( "Choice: " );
	fflush( stdout );
}


void DoSynchronize( void )
{
	long retcode;

	ConsoleClear();
	printf( "Sychronizing...\n" );

	retcode = pULDB->Synchronize();

	if( SQLE_NOERROR == retcode ) {
		printf( "\nSynchronization completed successfully.\n\n" );
	} else if ( SQLE_INVALID_LOGON == retcode ) {
		printf( "\nERROR: Bad login id or password.\n\n" );
	} else {
		printf( "\nERROR: synchronization returned: [%d]\n\n", retcode );
	}
	ConsolePressAnyKey();
}


void DoDisplayCustomers( void )
{
	TCHAR ch = '\0';

	while( ch != 'R' && ch != 'r' ) {
		PrintCustomers();
		PrintDisplayCustomersMenu();

		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch( ch ) {

		case 'R':   // return to main menu
			break;

		default:
			DoInvalidChoice( ch );
			break;
		}
	}
}

void PrintDisplayCustomersMenu( void )
{	
	printf( "\n\n\n\n" );
	printf( "------------------------------------------\n" );
	printf( "\n(R)eturn to Main Menu\n\n" );
	printf( "Choice: " );
	fflush( stdout );
}

void PrintCustomers( void )
{
	ConsoleClear();

	printf( "Customer Information\n" );
	printf( "------------------------------------------\n" );

	customer_struct customer;
	long retcode = pULDB->OpenCustomerCursor();

	switch( retcode ) {

	case SQLE_NOERROR:
		// Everything is okay!
		break;

	case SQLE_NOTFOUND:
		printf( "\n    ** Database contains no customers\n"
			"       You might try synchronizing...\n\n" );
		ConsolePressAnyKey();
		return;

	default:
		printf( "\nERROR: OpenCustomerCursor returned [%ld]\n\n", retcode );
		ConsolePressAnyKey();
		return;
	}
	
	bool hasAtLeastOne = false;
	retcode = pULDB->GetNextCustomerRecord( &customer );
	while ( retcode == SQLE_NOERROR ) {
		if ( !hasAtLeastOne ) {
			PrintCustomerHeader();
			hasAtLeastOne = true;
		}
		// We successfully fetched the new customer record.  Now let's
		// display it.
		//
		PrintCustomerRecord( &customer );
		retcode = pULDB->GetNextCustomerRecord( &customer );
	}

	if ( retcode != SQLE_NOTFOUND ) {
		// Some unexpected error occured.
		//
		printf( "\n\t** ERROR: fetch returned [%ld]\n", retcode );
		ConsolePressAnyKey();
	}

	if ( !hasAtLeastOne ) {
		printf( "\n    ** Database contains no customers\n"
			"       You might try synchronizing..." );
	}

	pULDB->CloseCustomerCursor();  // TODO: check that no error occurs here...
}

void PrintCustomerHeader( void )
{
	printf( "\n\n\tID\tNAME" );
}

void PrintCustomerRecord( customer_struct *customer )
{
	printf( "\n\n\t" );
	printf( "%ld", customer->customer_id );
	printf( "\t%s ", customer->first_name );
	printf( "%s", customer->last_name );
}


void DoInvalidChoice( TCHAR ch )
{
	// ignore space, tab, newline, etc...
	if( isspace( ch ) ) {
		return;
	}
	printf( "\n\n" );
	printf( "\tInvalid choice: [%c]\n\n", ch );

	ConsolePressAnyKey();
}


// --- Console Helper functions ---
//
void ConsolePressAnyKey( void )
{
	TCHAR ch;

	printf( "\nPress any key to continue..." );
	ch = getch();
	printf( "%c\n\n", ch );
}

bool ConsoleYesOrNo( TCHAR *msg )
{
	TCHAR ch;

	while( 1 ) {
		printf( "\n\n%s (y/n) ? ", msg );
		ch = toupper( getch() );
		printf( "%c\n", ch );

		switch( ch ) {

		case 'N':
			return FALSE;

		case 'Y':
			return TRUE;

		default:
			break;
		}
	}
}


// This function performs the same action as the DOS "cls" command
void ConsoleClear( void )
{
	HANDLE handle;
	CONSOLE_SCREEN_BUFFER_INFO info;
	COORD coord;
	DWORD dwTemp;

	SECURITY_ATTRIBUTES attr;
	attr.nLength = sizeof( attr );
	attr.lpSecurityDescriptor = NULL;
	attr.bInheritHandle = TRUE;

	coord.X = 0;
	coord.Y = 0;

	handle = CreateFile( TEXT( "CONOUT$" ),
			     GENERIC_READ | GENERIC_WRITE,
			     FILE_SHARE_READ | FILE_SHARE_WRITE,
			     &attr,
			     OPEN_EXISTING,
			     ( DWORD ) 0,
			     ( HANDLE ) 0 );

	if( handle != INVALID_HANDLE_VALUE ) {
		if( GetConsoleScreenBufferInfo( handle, &info ) ) {
			// Fill the console with the space character starting from 0,0
			FillConsoleOutputCharacter( handle, ( TCHAR ) ' ',
						    info.dwSize.X * info.dwSize.Y,
						    coord, &dwTemp );
			// Put the cursor back the 0,0 position
			SetConsoleCursorPosition( handle, coord );
		}
		CloseHandle( handle );
	}
}


int ConsoleExit( int exit_code )
{
	ConsolePressAnyKey();
	exit( exit_code );
}
